/*
 * SienaBandwidthGauge.java
 *
 * Created on November 13, 2001, 12:01 PM
 */

package edu.cmu.cs.able.gauges.Siena;
import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.probes.RemosProbe.*;
import java.net.*;
import java.util.*;
/** This class implements a BandwidthGauge, and uses RemosProbes
 * to determine the current bandwidth. It is assumed that a RemosProbe
 * is listening on the siena bus. The gauge type for this gauge is: <br>
 * BandwidthGauge = [<br>
 * &nbsp;&nbsp;type = edu.cmu.cs.able.gauges.Siena.SienaBandwidthGauge<br>
 * &nbsp;&nbsp;setup = [IP_addr1 : String = "localhost"; IP_addr2 = "localhost"; Frequency : long = 100]<br>
 * &nbsp;&nbsp;config = [Frequency : long = 100]<br>
 * &nbsp;&nbsp;values = [bandwidth : float]<br>
 * @author Bradley Schmerl
 * @version 1.1
 */
public class SienaBandwidthGauge extends edu.cmu.cs.able.gaugeInfrastructure.Siena.SienaGauge implements edu.cmu.cs.able.probes.RemosProbe.BandwidthProbeConsumer {

    /** The type of the BandwidthGauge (edu.cmu.cs.able.gauges.Siena.SienaBandwidthGauge)
     */    
    protected String gaugeType = "edu.cmu.cs.able.gauges.Siena.SienaBandwidthGauge";
    
    private BandwidthProbeConsumer remosProbeConsumer;
    private ProbeManagerHandle pmh;
    private ProbeHandle prbHndl;
    
    // Setup parameters
    /** The IP address of one endpoint to determine the bandwidth between.
     */    
    protected InetAddress ip1;
    /** The IP address of one endpoint to determine the bandwidth between.
     */    
    protected InetAddress ip2;
    /** How often this gauge should report the bandwidth.
     */    
    protected int frequency;
    
    /** The bandwidth that was currently measured from the probe.
     */    
    protected Double currentBW = null;
    /** The location of the siena bus server.
     */    
    public static String senp = null;
    
    /** Creates new SienaBandwidthGauge
     * @param gaugeID The ID of the gauge to create.
     * @param creatorID The ID of the gauge manager that is creating the gauge.
     * @param setupParams The value of the setup params, with items for IP_addr1, IP_addr2, and Frequency.
     * @param mappings The definition of the mapping between the bandwidth value and a model.
     */
    public SienaBandwidthGauge(GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        super (gaugeID, creatorID, setupParams, mappings);
       // setup default values
        try {
            ip1 = InetAddress.getByName ("localhost");
            ip2 = InetAddress.getByName ("localhost");
            sleepTime = Integer.MAX_VALUE; // the gauge waits for events from the probe before reporting
            frequency = 500;
        }
        catch (Exception e) {
            ip1 = null;
            ip2 = null;
        }
        
        try {
            for (int i = 0; i < setupParams.size(); i++) {
                if (setupParams.nameAt(i).equals("IP_addr1")) {
                    ip1 = InetAddress.getByName(setupParams.valueAt(i));
                }
                else if (setupParams.nameAt(i).equals("IP_addr2")) {
                    ip2 = InetAddress.getByName(setupParams.valueAt(i));
                }
                else if (setupParams.nameAt(i).equals("Frequency")) {
                    frequency = Integer.parseInt(setupParams.valueAt(i));
                }
            }
            
            boolean probesDeployed = deployProbes ();
            if (!probesDeployed) {
                finalizeCreation (creatorID, false);
            }
            else {
                finalizeCreation (creatorID, true);
            }
        }
        catch (Exception e) {
            // Needs to be some way to communicate to the gauge manager that there
            // was an error starting up!
            finalizeCreation (creatorID, false);
        }
    }

    /** Configures the gauge.
     * @param configParams The configuration parameters for this gauge.
     * @return
     */
    public boolean configure(StringPairVector configParams) {
        // The only valid configuration parameter is "Frequency"
        // This value will focus the remos probe to report with this frequency -
        // the gauge will report whenever there is a value sensed
        
        boolean validConfiguration = true;
        Integer frequency = new Integer (configParams.getValue ("Frequency"));
        if (frequency.intValue () <= 0) {
            validConfiguration = false;
        }
        else {
            this.frequency = frequency.intValue ();
            validConfiguration = prbHndl.focus (ip1, ip2, 1000/frequency.intValue ());
        }
        
        return validConfiguration;
    }
    
    /** Returns all the values that this gauge reports.
     * @param values Filled with an entry for bandwidth, with the value of the currentBW.
     * @return
     */
    public boolean queryAllValues(GaugeValueVector values) {
        values.addElement ("bandwidth", (String )mappings.get ("bandwidth"), currentBW.toString ());
        return true;
    }
    
    /** Returns the state of this gauge.
     * @param setupParams The parameters that the gauge was setup with.
     * @param configParams The current configuration of the gauge.
     * @param mappings The current mappings of the gauge.
     * @return
     */
    public boolean queryState(StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.addElement ("frequency", new Integer (frequency).toString ());
        
        Enumeration enum = this.mappings.keys ();
        while (enum.hasMoreElements ()) {
            String key = (String )enum.nextElement ();
            mappings.addElement (key, (String )this.mappings.get (key));
        }
        
        return true;
    }
    
    /** Returns the value of a particular value name
     * @param valueName The name of the value to return.
     * @return The gauge value
     */
    public String queryValue(String valueName) {
        if (valueName.equals ("bandwidth")) {
            if (currentBW != null) {
                return currentBW.toString ();
            }
            else {
                return null;
            }
        }
        else return null;
    }
    
    /** Reports a new value. This should be overridden by extenders of this gauge.
     */
    public void reportNewValue() {
    }
    
    private boolean deployProbes () {
        boolean probesDeployed = true;
        pmh = new ProbeManagerHandle (this, "myHandle", senp);
        prbHndl = pmh.deploy ();
        if (prbHndl == null || !prbHndl.activate (ip1, ip2, 1000/frequency)) {
            probesDeployed = false;
        }
        
        return probesDeployed;
    }
    
    // Probe consumer interface
    static final long EightMega = 1024*1024/8;
    
    /** The callback that implements BandwidthProbeConsumer. This is called when
     * the RemosProbe reports a value.
     * @param bw The bandwidth reported by the RemosProbe.
     */    
    public void sensed (double bw) {
        currentBW = new Double (bw/EightMega);
        gaugeBus.reportValue (getGaugeID (), "bandwidth", (String )mappings.get ("bandwidth"), currentBW.toString ());
    }
    
    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     * -senp <i>siena port</i><br>
     * [-debug]
     */    
    public static void main(String[] args) {
           
        String creatorID = null;
        String gaugeName = null;
        String gaugeType = null;
        String modelName = null;
        String modelType = null;
        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                } 
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-senp")) {
                senp = args [++i];
                edu.cmu.cs.able.gaugeInfrastructure.Siena.Initialization.initSiena (senp);
            }
            else if (args [i].equals ("-debug")) {
                edu.cmu.cs.able.gaugeInfrastructure.util.Global.debugFlag = true;
            }
            else {
                ok = false;
            }            
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;
   
       
        SienaBandwidthGauge gauge = new SienaBandwidthGauge (gid, creatorID, setupParams, mappings);
       
        
    }
    
    private static void printUsageMessage () {
        System.out.println ("Usage: SienaBandwidthGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -senp <sienaport>");
        
    }
    
    
}
